﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using emanual.Wpf.Controls;
using System.Diagnostics;

namespace TreeViewHelperTest
{
	public partial class MainWindow : Window
	{
		System.Text.Encoding FEncoding;

		public MainWindow()
		{
			InitializeComponent();
		}

		private void Window_PreviewKeyDown(object sender, KeyEventArgs e)
		{
			TreeViewItem item = treeView1.SelectedItem as TreeViewItem;

			if (item == null)
				return;

			switch (e.Key)
			{
				case Key.Left:
					if (Keyboard.Modifiers == ModifierKeys.Shift)
					{
						this.mnuEditLeft_Click(mnuEditLeft, null);
					}
					else if (Keyboard.Modifiers == ModifierKeys.Control)
					{
						this.mnuEditCollapse_Click(sender, null);
					}
					break;
				case Key.Right:
					if (Keyboard.Modifiers == ModifierKeys.Shift)
					{
						this.mnuEditLeft_Click(mnuEditRight, null);
					}
					else if (Keyboard.Modifiers == ModifierKeys.Control)
					{
						this.mnuEditExpand_Click(sender, null);
					}
					break;
				case Key.Up:
					if (Keyboard.Modifiers == ModifierKeys.Shift)
					{
						this.mnuEditLeft_Click(mnuEditUp, null);
					}
					else if (Keyboard.Modifiers == ModifierKeys.Control)
					{
						this.mnuEditShowUp_Click(sender, null);
					}

					break;
				case Key.Down:
					if (Keyboard.Modifiers == ModifierKeys.Shift)
					{
						this.mnuEditShowDown_Click(mnuEditDown, null);
					}
					else if (Keyboard.Modifiers == ModifierKeys.Control)
					{
						this.mnuEditShowDown_Click(sender, null);
					}
					break;
				case Key.N:
					if (Keyboard.Modifiers == ModifierKeys.Control)
					{
						this.mnuEditNew_Click(mnuEditDown, null);
					}
					break;
				case Key.Delete:
					if (Keyboard.Modifiers == ModifierKeys.Control)
					{
						this.mnuEditDelete_Click(mnuEditDown, null);
					}
					break;
			}
		}

		private void mnuFileOpenText_Click(object sender, RoutedEventArgs e)
		{
			var dlg = new Microsoft.Win32.OpenFileDialog();
			dlg.Filter = "TXT ファイル|*.txt";
			dlg.InitialDirectory = System.IO.Directory.GetCurrentDirectory();

			if (dlg.ShowDialog() == true)
			{
				FEncoding = emanual.Wpf.Utility.TextEncoding.GetEncodingFromFile(dlg.FileName);
				TreeViewHelper.LoadFromFile(treeView1, dlg.FileName, FEncoding);
				((TreeViewItem)treeView1.Items[0]).Focus();
			}
		}

		private void mnuFileOpenXml_Click(object sender, RoutedEventArgs e)
		{
			var dlg = new Microsoft.Win32.OpenFileDialog();
			dlg.Filter = "XML ファイル|*.xml";
			dlg.InitialDirectory = System.IO.Directory.GetCurrentDirectory();

			if (dlg.ShowDialog() == true)
			{
				TreeViewHelper.LoadFromXmlFile(treeView2, dlg.FileName, "Root/Item");
			}
		}

		private void mnuFileSaveText_Click(object sender, RoutedEventArgs e)
		{
			if (treeView1.Items.Count < 1)
				return;

			List<TreeItemData> list = TreeViewHelper.GetAllItems(treeView1);

			var writer = new System.IO.StreamWriter("output.txt", false, FEncoding);

			for (int i = 0; i < list.Count; ++i)
			{
				var item = list[i];
				string tab = new string('\t', item.Level);
				writer.WriteLine(tab + item.Header);
			}

			writer.Close();
		}

		private void mnuFileQuit_Click(object sender, RoutedEventArgs e)
		{
			this.Close();
		}

		private void mnuEditLeft_Click(object sender, RoutedEventArgs e)
		{
			string name = "";
			MenuItem menuItem = sender as MenuItem;

			if (menuItem == null)
			{
				Button button = sender as Button;

				if (button != null)
				{
					name = button.Name;
				}
			}
			else
				name = menuItem.Name;

			TreeViewItem item = treeView1.SelectedItem as TreeViewItem;

			if (item == null)
				return;

			switch (name)
			{
				case "mnuEditLeft":
				case "btnEditLeft":
					TreeViewHelper.MoveLeft(treeView1, item);
					break;
				case "mnuEditRight":
				case "btnEditRight":
					TreeViewHelper.MoveRight(treeView1, item);
					break;
				case "mnuEditUp":
				case "btnEditUp":
					TreeViewHelper.MoveUp(treeView1, item);
					break;
				case "mnuEditDown":
				case "btnEditDown":
					TreeViewHelper.MoveDown(treeView1, item);
					break;
			}
		}

		private void mnuEditNew_Click(object sender, RoutedEventArgs e)
		{
			TreeViewItem item = treeView1.SelectedItem as TreeViewItem;

			if (item == null)
				return;

			var newItem = new TreeItemData("追加項目");
			TreeViewHelper.InsertAfter(treeView1, item, newItem);
		}

		private void mnuEditDelete_Click(object sender, RoutedEventArgs e)
		{
			TreeViewItem item = treeView1.SelectedItem as TreeViewItem;

			if (item == null)
				return;

			TreeViewHelper.Remove(treeView1, item);
		}

		private void mnuEditShowUp_Click(object sender, RoutedEventArgs e)
		{
			TreeViewItem item = treeView1.SelectedItem as TreeViewItem;

			if (item == null)
				return;

			TreeViewItem newItem = TreeViewHelper.GetPrevVisibleItem(item) as TreeViewItem;

			if (newItem != null)
				newItem.IsSelected = true;
		}

		private void mnuEditShowDown_Click(object sender, RoutedEventArgs e)
		{
			TreeViewItem item = treeView1.SelectedItem as TreeViewItem;

			if (item == null)
				return;

			TreeViewItem newItem = TreeViewHelper.GetNextVisibleItem(true, item);

			if (newItem != null)
				newItem.IsSelected = true;
		}

		private void mnuEditExpand_Click(object sender, RoutedEventArgs e)
		{
			TreeViewItem item = treeView1.SelectedItem as TreeViewItem;

			if (item == null)
				return;

			item.IsExpanded = true;
		}

		private void mnuEditCollapse_Click(object sender, RoutedEventArgs e)
		{
			TreeViewItem item = treeView1.SelectedItem as TreeViewItem;

			if (item == null)
				return;

			item.IsExpanded = false;
		}

		private void mnuEditAllExpand_Click(object sender, RoutedEventArgs e)
		{
			if (treeView1.Items.Count < 1)
				return;

			TreeViewHelper.ExpandAll(treeView1);
		}

		private void mnuEditAllCollapse_Click(object sender, RoutedEventArgs e)
		{
			if (treeView1.Items.Count < 1)
				return;

			TreeViewHelper.CollapseAll(treeView1);
		}
	}
}
